﻿// Copyright (c) 2015, Coveo Solutions Inc.

using System;
using System.Collections.Generic;
using System.Linq;
using Coveo.AbstractLayer.Pipeline;
using Coveo.Framework.Fields.Config;
using Coveo.Framework.Processor;

namespace Coveo.Demos.Pipelines
{
    public class BeforeUpdateFieldSetProcessor : IProcessor<CoveoBeforeUpdateFieldSetPipelineArgs>
    {
        public void Process(CoveoBeforeUpdateFieldSetPipelineArgs p_Args)
        {
            // If the index being currently processed is the one associated with the 'master' database, then we process the numeric field.
            if (p_Args.IndexName == "sitecore_master_index") {
                SetNumericFieldsAsFacet(p_Args.Fields);
            }
        }

        private void SetNumericFieldsAsFacet(IEnumerable<FieldConfig> p_Fields)
        {
            // Here, we get the 'NumericField1' field (which is of type 'Number') and we set it as a facet, so that we will be able to create a Coveo Facet component based on it.
            // This is the same thing as setting the field facettable directly through the Coveo Search Provider's configuration file (isFacet="true").
            // This code is executed just before the Fields Set is processed by the Coveo Search Provider. 
            FieldConfig fieldConfig = p_Fields.FirstOrDefault(field => String.Equals(field.SitecoreFieldName, "NumericField1", StringComparison.InvariantCultureIgnoreCase));
            if (fieldConfig != null) {
                fieldConfig.IsFacet = true;
            }
        }
    }
}
