﻿// Copyright (c) 2015, Coveo Solutions Inc.

using Coveo.Framework.Processor;
using Coveo.SearchProvider.Pipelines;
using Sitecore.ContentSearch;
using Sitecore.Data.Fields;
using Sitecore.Data.Items;

namespace Coveo.Demos.Pipelines
{
    public class RelatedItemPreProcessor : IProcessor<CoveoItemProcessingPipelineArgs>
    {
        public void Process(CoveoItemProcessingPipelineArgs p_Args)
        {
            // In this example, our goal is to retrieve the media item related to the item being currently processed, and index it along with the current item.
            // The template of the current item contains a field named 'MediaItem' of type 'Droptree' which points to a specific item located in the media library.
            SitecoreIndexableItem indexableItem = p_Args.Item as SitecoreIndexableItem;
            IIndexableBuiltinFields builtInFields = indexableItem;
            if (indexableItem != null) {
                Item item = indexableItem.Item;
                // If the item being currently processed is of the expected type (template = 'Sample Item V3'), we process it.
                if (builtInFields.TemplateId.ToString().Equals("{41013191-AB12-4074-BFF0-3861AA2148AC}")) {
                    // Since this is a pre-processor, the Coveo Search Provider has not yet processed the current item, so we only have the raw Sitecore item (i.e. SitecoreIndexableItem)
                    // in our hands.
                    //
                    // Here, we retrieve the related media library item and we append it to the list of output items.
                    // p_Args.OutputItem already contains the original item.
                    Field mediaItemField = item.Fields["MediaItem"];
                    Sitecore.Data.Database database = Sitecore.Configuration.Factory.GetDatabase(builtInFields.Database);
                    Item relatedMediaItem = database.GetItem(mediaItemField.Value);
                    p_Args.OutputItems.Add(new SitecoreIndexableItem(relatedMediaItem));
                }
            }
        }
    }
}
