﻿// Copyright (c) 2015, Coveo Solutions Inc.

using System.Collections.Generic;
using Coveo.AbstractLayer.RepositoryItem;
using Coveo.Framework.Processor;
using Coveo.SearchProvider.Pipelines;
using Sitecore.ContentSearch;
using Sitecore.Data.Fields;
using Sitecore.Data.Items;

namespace Coveo.Demos.Pipelines
{
    public class TranslatedItemPostProcessor : IProcessor<CoveoPostItemProcessingPipelineArgs>
    {
        public void Process(CoveoPostItemProcessingPipelineArgs p_Args)
        {
            // In this example, our goal is to manually translate a specific item from English to French, and index its translated version as a separate item.
            SitecoreIndexableItem indexableItem = p_Args.Item as SitecoreIndexableItem;
            IIndexableBuiltinFields builtInFields = indexableItem;
            if (indexableItem != null) {
                Item item = indexableItem.Item;
                // If the item being currently processed is of the expected type (template = Sample Item), we process it.
                if (builtInFields.TemplateId.ToString().Equals("{76036F5E-CBCE-46D1-AF0A-4143F9B557AA}")) {
                    // If the 'Text' field has the expected value, it means that the current item is indeed the one that we are interested in.
                    Field textField = item.Fields["Text"];
                    if (textField.Value == "<p>This is a test!</p>") {
                        // The value of the original item's UniqueId property has the following format: sitecore://{database}/{item_id}?lang={language}&ver={version}.
                        // But our new item must have a different UniqueId value to be indexed by Coveo, so we change the values of its language and version parameters.
                        string newUniqueId = p_Args.CoveoItem.UniqueId.Replace("lang=en", "lang=fr").Replace("ver=1", "ver=2");

                        // To create a new item, we clone the original one and we change the properties and values that we are interested in.
                        CoveoIndexableItem newItem = new CoveoIndexableItem {
                            BinaryData = p_Args.CoveoItem.BinaryData,
                            BinaryDataMimeType = p_Args.CoveoItem.BinaryDataMimeType,
                            BinaryDataPath = p_Args.CoveoItem.BinaryDataPath,
                            ClickableUri = p_Args.CoveoItem.ClickableUri,
                            FileName = p_Args.CoveoItem.FileName,
                            HasSubItems = p_Args.CoveoItem.HasSubItems,
                            Id = p_Args.CoveoItem.Id,
                            IsDeletedItem = p_Args.CoveoItem.IsDeletedItem,
                            ModifiedDate = p_Args.CoveoItem.ModifiedDate,
                            Parent = p_Args.CoveoItem.Parent,
                            ParentId = p_Args.CoveoItem.ParentId,
                            Path = p_Args.CoveoItem.Path,
                            Permissions = p_Args.CoveoItem.Permissions,
                            PrintablePath = p_Args.CoveoItem.PrintablePath,
                            Title = "Item Exemple",
                            UniqueId = newUniqueId,
                            Metadata = new Dictionary<string, object>(p_Args.CoveoItem.Metadata)
                        };

                        // Here, we:
                        // -Manually translate the original title to French.
                        // -Change the value of a few metadata fields to reflect the differences in our translated item.
                        newItem.SetMetadata("Text", "Ceci est un test!");
                        newItem.SetMetadata("Language", "fr");
                        newItem.SetMetadata("Languages", "fr");
                        newItem.SetMetadata("Version", "2");

                        // Since we are using a post-processing pipeline, computed fields such as ParsedLanguage have already been processed.
                        // So here we change its value to 'french'.
                        newItem.SetMetadata("parsedlanguage", "french");

                        // OutputCoveoItems always contains the original item as its first item. It's actually the same instance of CoveoIndexableItem as in p_Args.CoveoItem, so if you want to
                        // modify the original item, you can do it either through p_Args.CoveoItem or p_Args.OutputCoveoItems.
                        p_Args.OutputCoveoItems.Add(newItem);
                    }
                }
            }
        }
    }
}
